@@ -1,14 +1,17 @@
 """Matter Client implementation."""
+
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Callable
 import logging
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, Callable, Dict, Final, Optional, cast
+from typing import TYPE_CHECKING, Any, Final, Optional, cast
 import uuid
 
 from aiohttp import ClientSession
 from chip.clusters import Objects as Clusters
+from chip.clusters.Types import NullValue
 
 from matter_server.common.errors import ERROR_MAP, NodeNotExists
 
@@ -60,8 +63,8 @@ class MatterClient:
         """Initialize the Client class."""
         self.connection = MatterClientConnection(ws_server_url, aiohttp_session)
         self.logger = logging.getLogger(__package__)
-        self._nodes: Dict[int, MatterNode] = {}
-        self._result_futures: Dict[str, asyncio.Future] = {}
+        self._nodes: dict[int, MatterNode] = {}
+        self._result_futures: dict[str, asyncio.Future] = {}
         self._subscribers: dict[str, list[Callable[[EventType, Any], None]]] = {}
         self._stop_called: bool = False
         self._loop: asyncio.AbstractEventLoop | None = None
@@ -255,15 +258,54 @@ class MatterClient:
             await self.send_command(APICommand.PING_NODE, node_id=node_id),
         )
 
+    async def get_node_ip_addresses(
+        self, node_id: int, prefer_cache: bool = True, scoped: bool = False
+    ) -> list[str]:
+        """Return the currently known (scoped) IP-adress(es)."""
+        if TYPE_CHECKING:
+            assert self.server_info is not None
+        if self.server_info.schema_version >= 8:
+            return cast(
+                list[str],
+                await self.send_command(
+                    APICommand.GET_NODE_IP_ADRESSES,
+                    require_schema=8,
+                    node_id=node_id,
+                    prefer_cache=prefer_cache,
+                    scoped=scoped,
+                ),
+            )
+        # alternative method of fetching ip addresses by enumerating NetworkInterfaces
+        node = self.get_node(node_id)
+        attribute = Clusters.GeneralDiagnostics.Attributes.NetworkInterfaces
+        network_interface: Clusters.GeneralDiagnostics.Structs.NetworkInterface
+        ip_addresses: list[str] = []
+        for network_interface in node.get_attribute_value(
+            0, cluster=None, attribute=attribute
+        ):
+            # ignore invalid/non-operational interfaces
+            if not network_interface.isOperational:
+                continue
+            # enumerate ipv4 and ipv6 addresses
+            for ipv4_address_hex in network_interface.IPv4Addresses:
+                ipv4_address = convert_ip_address(ipv4_address_hex)
+                ip_addresses.append(ipv4_address)
+            for ipv6_address_hex in network_interface.IPv6Addresses:
+                ipv6_address = convert_ip_address(ipv6_address_hex, True)
+                ip_addresses.append(ipv6_address)
+            break
+        return ip_addresses
+
     async def node_diagnostics(self, node_id: int) -> NodeDiagnostics:
         """Gather diagnostics for the given node."""
+        # pylint: disable=too-many-statements
         node = self.get_node(node_id)
+        ip_addresses = await self.get_node_ip_addresses(node_id)
         # grab some details from the first (operational) network interface
         network_type = NetworkType.UNKNOWN
         mac_address = None
         attribute = Clusters.GeneralDiagnostics.Attributes.NetworkInterfaces
         network_interface: Clusters.GeneralDiagnostics.Structs.NetworkInterface
-        ip_addresses: list[str] = []
         for network_interface in node.get_attribute_value(
             0, cluster=None, attribute=attribute
         ):
@@ -289,13 +331,6 @@ class MatterClient:
                 # unknown interface: ignore
                 continue
             mac_address = convert_mac_address(network_interface.hardwareAddress)
-            # enumerate ipv4 and ipv6 addresses
-            for ipv4_address_hex in network_interface.IPv4Addresses:
-                ipv4_address = convert_ip_address(ipv4_address_hex)
-                ip_addresses.append(ipv4_address)
-            for ipv6_address_hex in network_interface.IPv6Addresses:
-                ipv6_address = convert_ip_address(ipv6_address_hex, True)
-                ip_addresses.append(ipv6_address)
             break
         # get thread/wifi specific info
         node_type = NodeType.UNKNOWN
@@ -304,7 +339,10 @@ class MatterClient:
             thread_cluster: Clusters.ThreadNetworkDiagnostics = node.get_cluster(
                 0, Clusters.ThreadNetworkDiagnostics
             )
-            network_name = thread_cluster.networkName
+            if isinstance(thread_cluster.networkName, bytes):
+                network_name = thread_cluster.networkName.decode("utf-8")
+            elif thread_cluster.networkName != NullValue:
+                network_name = thread_cluster.networkName
             # parse routing role to (diagnostics) node type
             if (
                 thread_cluster.routingRole
@@ -322,17 +360,49 @@ class MatterClient:
             ):
                 node_type = NodeType.END_DEVICE
         elif network_type == NetworkType.WIFI:
-            wifi_cluster: Clusters.WiFiNetworkDiagnostics = node.get_cluster(
-                0, Clusters.WiFiNetworkDiagnostics
-            )
-            if wifi_cluster and wifi_cluster.bssid:
-                network_name = wifi_cluster.bssid
             node_type = NodeType.END_DEVICE
+        # use lastNetworkID from NetworkCommissioning cluster as fallback to get the network name
+        # this allows getting the SSID as the wifi diagnostics cluster only has the BSSID
+        last_network_id: bytes | str | None
+        if not network_name and (
+            last_network_id := node.get_attribute_value(
+                0,
+                cluster=None,
+                attribute=Clusters.NetworkCommissioning.Attributes.LastNetworkID,
+            )
+        ):
+            if isinstance(last_network_id, bytes):
+                network_name = last_network_id.decode("utf-8")
+            elif last_network_id != NullValue:
+                network_name = last_network_id
+        # last resort to get the (wifi) networkname;
+        # enumerate networks on the NetworkCommissioning cluster
+        networks: list[Clusters.NetworkCommissioning.Structs.NetworkInfoStruct]
+        if not network_name and (
+            networks := node.get_attribute_value(
+                0,
+                cluster=None,
+                attribute=Clusters.NetworkCommissioning.Attributes.Networks,
+            )
+        ):
+            for network in networks:
+                if not network.connected:
+                    continue
+                if isinstance(network.networkID, bytes):
+                    network_name = network.networkID.decode("utf-8")
+                    break
+                if network.networkID != NullValue:
+                    network_name = network.networkID
+                    break
         # override node type if node is a bridge
         if node.node_data.is_bridge:
             node_type = NodeType.BRIDGE
         # get active fabrics for this node
         active_fabrics = await self.get_matter_fabrics(node_id)
+        # get active fabric index
+        fabric_index = node.get_attribute_value(
+            0, None, Clusters.OperationalCredentials.Attributes.CurrentFabricIndex
+        )
         return NodeDiagnostics(
             node_id=node_id,
             network_type=network_type,
@@ -342,6 +412,7 @@ class MatterClient:
             mac_address=mac_address,
             available=node.available,
             active_fabrics=active_fabrics,
+            active_fabric_index=fabric_index,
         )
 
     async def send_device_command(
@@ -375,24 +446,26 @@ class MatterClient:
         self,
         node_id: int,
         attribute_path: str,
-    ) -> Any:
-        """Read attribute(s) on a node."""
-        return await self.send_command(
+    ) -> dict[str, Any]:
+        """Read one or more attribute(s) on a node by specifying an attributepath."""
+        updated_values = await self.send_command(
             APICommand.READ_ATTRIBUTE,
             require_schema=4,
             node_id=node_id,
             attribute_path=attribute_path,
         )
+        if not isinstance(updated_values, dict):
+            # can happen is the server is running schema < 8
+            return {attribute_path: updated_values}
+        return cast(dict[str, Any], updated_values)
 
     async def refresh_attribute(
         self,
         node_id: int,
         attribute_path: str,
-    ) -> Any:
+    ) -> None:
         """Read attribute(s) on a node and store the updated value(s)."""
         updated_values = await self.read_attribute(node_id, attribute_path)
-        if not isinstance(updated_values, dict):
-            updated_values = {attribute_path: updated_values}
         for attr_path, value in updated_values.items():
             self._nodes[node_id].update_attribute(attr_path, value)
 
